# -*- encoding: utf-8 -*-
from aimaestro.abcglobal.key import *
from aimaestro.plugins._mongodb.mongo_client import MongoClient

__all__ = [
    'query_service_list',
    'service_insert_one',
    'update_work_max_process',
    'query_worker_running_number',
    'worker_push_process',
    'worker_pull_process'
]


class ServiceAPI(MongoClient):
    """
        DatabaseServices class is used to manage database services - related operations.
    """
    _table_name = TABLE_NAME_SERVICES


def query_service_list(query: dict, field: dict, limit: int, skip_no: int):
    """
    Query the list of services from the database.
    Args:
        query (dict): The query criteria.
        field (dict): The fields to include in the result.
        limit (int): The maximum number of results to return.
        skip_no (int): The number of results to skip.
    Returns:
        list: The list of services.
    """
    _database_services = ServiceAPI()

    return _database_services.query_list_sort(query=query, field=field, limit=limit, skip_no=skip_no)


def service_insert_one(query: dict, data: dict):
    """
    Insert a single document into the collection.

    Args:
        data (dict): A dictionary containing the data to insert.
        query (dict): A dictionary specifying the query criteria.
    """
    _database_services = ServiceAPI()
    _database_services.update_many(query=query, update_data=data, upsert=True)


def query_worker_running_number(query: dict):
    """
    Query the number of running workers based on the given query.
    Args:
        query (dict): A dictionary containing the query criteria.
    Returns:
        tuple: A tuple containing the number of running workers and the maximum number of workers.
    """
    _database_services = ServiceAPI()
    data = _database_services.query_all(
        query=query,
        field={'_id': 0, KEY_WORKER_RUN_PROCESS: 1, KEY_WORKER_MAX_PROCESS: 1}
    )
    return len(data[0].get(KEY_WORKER_RUN_PROCESS)), data[0].get(KEY_WORKER_MAX_PROCESS)


def update_work_max_process(worker_name: str, worker_ipaddr: str, worker_max_process: int):
    """
    Update the maximum number of processes for a worker identified by its name and IP address.

    Args:
        worker_name (str): The name of the worker.
        worker_ipaddr (str): The IP address of the worker.
        worker_max_process (int): The new maximum number of processes for the worker.

    Returns:
        None
    """
    _database_services = ServiceAPI()
    _database_services.update_many(
        query={
            KEY_WORKER_NAME: worker_name,
            KEY_WORKER_IPADDR: worker_ipaddr
        },
        update_data={
            KEY_WORKER_MAX_PROCESS: worker_max_process
        }
    )


def worker_push_process(worker_name: str, worker_ipaddr: str, worker_pid: int):
    """
    Update the maximum number of processes for a worker identified by its name and IP address.
    Args:
        worker_name (str): The name of the worker.
        worker_ipaddr (str): The IP address of the worker.
        worker_pid (int): The new maximum number of processes for the worker.
    Returns:
        None
    """
    _database_services = ServiceAPI()
    _database_services.push_one(
        query={
            KEY_WORKER_NAME: worker_name,
            KEY_WORKER_IPADDR: worker_ipaddr
        },
        update_data={
            KEY_WORKER_RUN_PROCESS: worker_pid
        }
    )


def worker_pull_process(worker_name: str, worker_ipaddr: str, worker_pid: int):
    """
    Update the maximum number of processes for a worker identified by its name and IP address.
    Args:
        worker_name (str): The name of the worker.
        worker_ipaddr (str): The IP address of the worker.
        worker_pid (int): The new maximum number of processes for the worker.
    Returns:
        None
    """

    _database_services = ServiceAPI()
    _database_services.pull_one(
        query={
            KEY_WORKER_NAME: worker_name,
            KEY_WORKER_IPADDR: worker_ipaddr
        },
        update_data={
            KEY_WORKER_RUN_PROCESS: worker_pid
        }
    )



# -*- encoding: utf-8 -*-

import abc

from aimaestro.taskflux.logger.logger import logger
from aimaestro.taskflux.interface.interface import databases_send_message, databases_submit_task

__all__ = ['ServiceConstructor', 'WorkerConstructor']


class Meta(abc.ABC):

    @property
    @abc.abstractmethod
    def submit_task(self): ...

    @property
    @abc.abstractmethod
    def send_message(self): ...

    @property
    @abc.abstractmethod
    def logger(self): ...

    @property
    @abc.abstractmethod
    def name(self): ...


class ServiceMeta(Meta):

    @property
    @abc.abstractmethod
    def service_name(self): ...

    @property
    @abc.abstractmethod
    def service_version(self): ...

    @property
    @abc.abstractmethod
    def service_ipaddr(self): ...


class WorkerMeta(Meta):

    @property
    @abc.abstractmethod
    def worker_name(self): ...

    @property
    @abc.abstractmethod
    def worker_version(self): ...

    @property
    @abc.abstractmethod
    def worker_ipaddr(self): ...

    @abc.abstractmethod
    def run(self, data): ...


class ServiceConstructor(ServiceMeta):
    """
    ServiceConstructor is a class that represents a constructor for a service.
    It contains attributes such as name, logger, service_name, service_ipaddr, service_version, and functions.
    """
    functions: list = []
    name: str = None
    service_name: str = None
    service_ipaddr: str = None
    service_version: str = None

    logger: logger = None
    submit_task: databases_submit_task = None
    send_message: databases_send_message = None

    @classmethod
    def setattr(cls, name, value):
        setattr(cls, name, value)

    def __call__(self):
        return self


class WorkerConstructor(WorkerMeta):
    """
    WorkerConstructor is a class that represents a constructor for a worker.
    It contains attributes such as name, logger, worker_name, worker_ipaddr, worker_version, and functions.
    It also contains a method called run, which takes in a body and runs the worker with the given body.
    """
    functions: list = []

    worker_name: str = None
    worker_ipaddr: str = None
    worker_version: str = None

    name: str = None
    logger: logger = None
    submit_task: databases_submit_task = None
    send_message: databases_send_message = None

    @classmethod
    def setattr(cls, name, value):
        setattr(cls, name, value)

    def run(self, body):
        """
        Runs the worker with the given body.
        Args:
            body (dict): The body of the worker.
        """

    def __call__(self):
        return self



# -*- encoding: utf-8 -*-
from aimaestro.abcglobal.key import *
from aimaestro.plugins._mongodb.mongo_client import MongoClient

__all__ = [
    'query_user_list',
    'query_user_only',
    'insert_user'
]


class UserAPI(MongoClient):
    """
        DatabaseTasks class is used to manage database task - related operations.
    """
    _table_name = TABLE_NAME_USERS


def query_user_list(query: dict, field: dict, limit: int, skip_no: int):
    """
    Query the list of tasks from the database.
    Args:
        query (dict): The query criteria.
        field (dict): The fields to include in the result.
        limit (int): The maximum number of results to return.
        skip_no (int): The number of results to skip.
    Returns:
        list: The list of services.
    """
    _database_users = UserAPI()
    return _database_users.query_list_sort(query=query, field=field, limit=limit, skip_no=skip_no)


def query_user_only(query: dict, field: dict):
    _database_users = UserAPI()
    return _database_users.query_one(query=query, field=field)


def insert_user(data: dict):
    _database_users = UserAPI()
    return _database_users.insert_data(data=data)



# -*- encoding: utf-8 -*-
import sys
import json
import base64

from aimaestro.abcglobal.key import *
from aimaestro.taskflux.generateId.snowflake import snowflake_id

__all__ = [
    'load_config',
    'task_required_field_check'
]


def _blank_dictionary_value_processing(data: dict, key_name: str, is_bool: bool = False):
    """
    Process the value of a dictionary key_name.
    Args:
        data (dict): The dictionary containing the key_name-value pair.
        key_name (str): The key_name to process.
        is_bool (bool, optional): Indicates if the value should be treated as a boolean. Defaults to False.
    Returns:
        bool: True if the key_name exists in the dictionary and the value is not None or an empty string.
    """

    if key_name in data and data[key_name] is not None and data[key_name] != '':
        if is_bool is False:
            return True

        if type(data[key_name]) is str and data[key_name].lower() == 'true':
            return True

        if type(data[key_name]) is bool and data[key_name] is True:
            return True
    return False


def _config_default_value_processing(config: dict):
    """
    Process the default values in the configuration dictionary.
    Args:
        config (dict): The configuration dictionary.
    Returns:
        dict: The configuration dictionary with processed default values.
    """

    if KEY_ADMIN_USERNAME not in config:
        config[KEY_ADMIN_USERNAME] = DEFAULT_VALUE_USERNAME

    if KEY_ADMIN_PASSWORD not in config:
        config[KEY_ADMIN_PASSWORD] = DEFAULT_VALUE_PASSWORD

    if KEY_DEFAULT_SCHEDULE_TIME not in config:
        config[KEY_DEFAULT_SCHEDULE_TIME] = DEFAULT_VALUE_SCHEDULE_TIME

    return config


def task_required_field_check(message: dict):
    """
    Check if the required fields are present in the task message.
    Args:
        message (dict): The task message.
    Raises:
        Exception: If any of the required fields are missing.
    Returns:
        message (dict): The task message.
    """

    if _blank_dictionary_value_processing(data=message, key_name=KEY_TASK_ID) is False:
        message[KEY_TASK_ID] = snowflake_id()

    return message


def load_config(encoded_config):
    """
    Decode and load the encoded configuration string.

    Args:
        encoded_config (str): The encoded configuration string.

    Returns:
        dict: The decoded configuration dictionary.
    """
    try:
        encoded_bytes = encoded_config.encode('utf-8')
        decoded_bytes = base64.b64decode(encoded_bytes)
        decoded_string = decoded_bytes.decode('utf-8')
        return _config_default_value_processing(json.loads(decoded_string))
    except Exception as e:
        print(f"Error loading config: {e}")
        sys.exit(1)



# -*- encoding: utf-8 -*-

KEY_ROOT_PATH = 'ROOT_PATH'
KEY_PROJECT_NAME = 'aimaestro'
TABLE_NAME_TASKS = 'task_list'
TABLE_NAME_USERS = 'user_list'
TABLE_NAME_NODES = 'server_list'
TABLE_NAME_SERVICES = 'service_list'

TABLE_NAME_DIALOGUES = 'dialogues'
TABLE_NAME_DIALOGUE_MESSAGE = 'dialogue_message'

KEY_LOGS_PATH = 'logs'
KEY_LOGS_FILENAME = 'filename'

KEY_RPC_PROXY_NAME = 'rpc'
KEY_RPC_METHOD_NAME = 'call'

KEY_RABBITMQ_URI = 'AMQP_URI'
KEY_RABBITMQ_CONFIG = 'RABBITMQ_CONFIG'
KEY_SYSTEM_SERVICE_NAME = 'proxy_system_server'

KEY_MONGO_CONFIG = 'MONGODB_CONFIG'

KEY_TASK_BODY = 'body'
KEY_TASK_ID = 'task_id'
KEY_TASK_WEIGHT = 'weight'
KEY_TASK_STATUS = 'status'
KEY_TASK_SOURCE_ID = 'source_id'
KEY_TASK_QUEUE_NAME = 'queue_name'
KEY_TASK_IS_SUB_TASK = 'is_subtask'
KEY_TASK_IS_SUB_TASK_ALL_FINISH = 'is_subtask_all_finish'

KEY_TASK_WAIT_STATUS = 'wait'
KEY_TASK_SEND_STATUS = 'send'
KEY_TASK_STOP_STATUS = 'stop'
KEY_TASK_ERROR_STATUS = 'error'
KEY_TASK_RUN_STATUS = 'running'
KEY_TASK_SUCCESS_STATUS = 'success'
KEY_TASK_ERROR_MESSAGE = 'error_message'

KEY_TASK_END_TIME = 'end_time'
KEY_TASK_START_TIME = 'start_time'
KEY_TASK_CREATE_TIME = 'create_time'

KEY_NAME = 'name'
KEY_SERVICE_PID = 'service_pid'
KEY_SERVICE_NAME = 'service_name'
KEY_SERVICE_IPADDR = 'service_ipaddr'
KEY_SERVICE_VERSION = 'service_version'
KEY_SERVICE_FUNCTIONS = 'service_functions'

KEY_WORKER_PID = 'worker_pid'
KEY_WORKER_NAME = 'worker_name'
KEY_WORKER_IPADDR = 'worker_ipaddr'
KEY_WORKER_VERSION = 'worker_version'
KEY_WORKER_FUNCTIONS = 'worker_functions'
KEY_WORKER_MAX_PROCESS = 'worker_max_process'
KEY_WORKER_RUN_PROCESS = 'worker_run_process'

KEY_NODE_IPADDR = 'ipaddr'

KEY_ADMIN_USERNAME = 'ADMIN_USERNAME'
KEY_ADMIN_PASSWORD = 'ADMIN_PASSWORD'
KEY_HTTP_SERVER_FORK = 'HTTP_SERVER_FORK'

KEY_DEFAULT_SCHEDULE_TIME = 'DEFAULT_SCHEDULE_TIME'

KEY_FUNCTION_SELF = 'self'
KEY_FUNCTION_RPC = 'RpcFunction'
KEY_FUNCTION_WORKER = 'WorkerFunction'
KEY_FUNCTION_PARAM_NAME = 'param_name'
KEY_FUNCTION_PARAM_DEFAULT_VALUE = 'default_value'

KEY_CIPHER_CIPHERTEXT = 'CIPHERTEXT'
KEY_CIPHER_PRIVATE_KEY = 'PRIVATE_KEY'
KEY_CIPHER_PUBLIC_KEY = 'PUBLIC_KEY'

KEY_RPC_CALL_TIMEOUT = 'RPC_CALL_TIMEOUT'

SOCKET_BIND_PORT = 80
SOCKET_BIND_IP = '8.8.8.8'
SOCKET_SHUTDOWN_SLEEP = 2

DEFAULT_VALUE_TIMEZONE = 'Asia/Shanghai'
DEFAULT_VALUE_TIME_FMT = '%Y%m%d%H%M%S'

DEFAULT_VALUE_LOGS_SUFFIX = "%Y-%m-%d.log"
DEFAULT_VALUE_LOGS_FMT = '%(asctime)s - %(levelname)s - [%(filename)s:%(lineno)d] %(message)s'

DEFAULT_VALUE_TASK_WEIGHT = 1
DEFAULT_VALUE_HTTP_SERVER_FORK = False

DEFAULT_VALUE_USERNAME = 'scheduleAdmin'
DEFAULT_VALUE_PASSWORD = 'scheduleAdminPassword'

DEFAULT_VALUE_SCHEDULE_TIME = 10
DEFAULT_VALUE_RPC_CALL_TIMEOUT = 30
DEFAULT_VALUE_RABBITMQ_URI = 'amqp://scheduleAdmin:scheduleAdminPassword@127.0.0.1:5672'
DEFAULT_VALUE_MONGODB_URI = 'mongodb://scheduleAdmin:scheduleAdminPassword@127.0.0.1:27017'



# -*- encoding: utf-8 -*-
from aimaestro.plugins._mongodb.mongo_client import initialization_mongo
from aimaestro.plugins._mongodb.user import *
from aimaestro.plugins._mongodb.task import *
from aimaestro.plugins._mongodb.node import *
from aimaestro.plugins._mongodb.service import *
from aimaestro.plugins._mongodb.aiagent import *



# -*- encoding: utf-8 -*-
from aimaestro.abcglobal.key import *
from aimaestro.plugins._mongodb.mongo_client import MongoClient

__all__ = [
    'query_node_list',
    'node_insert_one'
]


class NodeAPI(MongoClient):
    """
        DatabaseNodes class is used to manage database nodes.
    """

    _table_name = TABLE_NAME_NODES


def query_node_list(query: dict, field: dict, limit: int, skip_no: int):
    """
    Query the list of nodes from the database.
    Args:
        query (dict): The query criteria.
        field (dict): The fields to include in the result.
        limit (int): The maximum number of results to return.
        skip_no (int): The number of results to skip.
    Returns:
        list: The list of services.
    """
    _database_nodes = NodeAPI()
    return _database_nodes.query_list_sort(query=query, field=field, limit=limit, skip_no=skip_no)


def node_insert_one(query: dict, data: dict):
    """
    Insert a single document into the collection.

    Args:
        data (dict): A dictionary containing the data to insert.
        query (dict): A dictionary specifying the query criteria.
    """
    _database_nodes = NodeAPI()
    _database_nodes.update_many(query=query, update_data=data, upsert=True)



# -*- encoding: utf-8 -*-
from aimaestro.abcglobal.key import *
from aimaestro.taskflux.rpc_proxy.client import RpcClient

__all__ = ['initialization_rpc_proxy', 'remote_call', 'generate_unique', 'proxy_call']

_RPC_CONFIG = {
    KEY_RABBITMQ_URI: DEFAULT_VALUE_RABBITMQ_URI,
    KEY_RPC_CALL_TIMEOUT: DEFAULT_VALUE_RPC_CALL_TIMEOUT
}


def initialization_rpc_proxy(config: dict):
    """
    Initialize the logger with the given configuration.
    Args:
        config (dict): A dictionary containing the configuration.
    """
    global _RPC_CONFIG
    _RPC_CONFIG[KEY_RABBITMQ_URI] = config.get(KEY_RABBITMQ_CONFIG, DEFAULT_VALUE_RABBITMQ_URI)
    _RPC_CONFIG[KEY_RPC_CALL_TIMEOUT] = config.get(DEFAULT_VALUE_RPC_CALL_TIMEOUT, DEFAULT_VALUE_RPC_CALL_TIMEOUT)


def generate_unique():
    """
    Generates a unique identifier.
    Returns:
        str: The generated identifier.
    """
    client = RpcClient(config=_RPC_CONFIG)
    service_name = '{}_{}_{}'.format(KEY_PROJECT_NAME, KEY_SYSTEM_SERVICE_NAME, 'task_distribution')
    data = client.call(service_name=service_name, method_name='generate_id')
    return data


def remote_call(service_name: str, method_name: str, **params):
    """
    Makes a remote procedure call to the specified service and method with the given parameters.

    Args:
        service_name (str): The name of the service to call.
        method_name (str): The name of the method to call.
        **params: Arbitrary keyword arguments to pass to the method.

    Returns:
        Any: The result of the remote procedure call.
    """
    client = RpcClient(config=_RPC_CONFIG)
    data = client.call(
        service_name=service_name,
        method_name=method_name,
        **params
    )
    return data


def proxy_call(service_name: str, method_name: str, **params):
    """
    Makes a remote procedure call to the specified service and method with the given parameters.

    Args:
        service_name (str): The name of the service to call.
        method_name (str): The name of the method to call.
        **params: Arbitrary keyword arguments to pass to the method.

    Returns:
        Any: The result of the remote procedure call.
    """
    client = RpcClient(config=_RPC_CONFIG)
    _name = '{}_{}'.format(KEY_PROJECT_NAME, service_name)

    data = client.call(
        service_name=_name,
        method_name=method_name,
        **params
    )
    return data



# -*- encoding: utf-8 -*-
from aimaestro.abcglobal.key import *
from aimaestro.plugins._mongodb.mongo_client import MongoClient

__all__ = [
    'query_task_list',
    'task_insert_one',
    'query_task_status_by_task_id',
    'task_stop',
    'task_retry',
    'query_run_task'
]


class TaskAPI(MongoClient):
    """
        DatabaseTasks class is used to manage database task - related operations.
    """
    _table_name = TABLE_NAME_TASKS


def query_task_list(query: dict, field: dict, limit: int, skip_no: int):
    """
    Query the list of tasks from the database.
    Args:
        query (dict): The query criteria.
        field (dict): The fields to include in the result.
        limit (int): The maximum number of results to return.
        skip_no (int): The number of results to skip.
    Returns:
        list: The list of services.
    """
    _database_tasks = TaskAPI()
    return _database_tasks.query_list_sort(query=query, field=field, limit=limit, skip_no=skip_no)


def task_insert_one(query: dict, data: dict):
    """
    Insert a single document into the collection.

    Args:
        data (dict): A dictionary containing the data to insert.
        query (dict): A dictionary specifying the query criteria.
    """
    _database_tasks = TaskAPI()
    _database_tasks.update_many(query=query, update_data=data, upsert=True)


def query_task_status_by_task_id(task_id: str):
    """
    Retrieve the task status by the given task ID.

    Args:
        task_id (str): The unique identifier of the task.

    Returns:
        dict: The first document containing the task status information.
    """
    _database_tasks = TaskAPI()
    data = _database_tasks.query_all(
        query={KEY_TASK_ID: task_id},
        field={
            '_id': 0, KEY_TASK_STATUS: 1,
            '{}.{}'.format(KEY_TASK_BODY, KEY_TASK_IS_SUB_TASK_ALL_FINISH): 1
        }
    )
    data = [i for i in data]
    return data[0]


def task_stop(task_id: str):
    """
    Stop a task by the given task ID.
    Args:
        task_id (str): The unique identifier of the task.
    Returns:
        None
    """
    _database_tasks = TaskAPI()
    _database_tasks.update_many(
        query={KEY_TASK_ID: task_id}, update_data={KEY_TASK_STATUS: KEY_TASK_STOP_STATUS})
    _database_tasks.update_many(
        query={KEY_TASK_SOURCE_ID: task_id}, update_data={KEY_TASK_STATUS: KEY_TASK_STOP_STATUS})


def task_retry(task_id: str):
    """
    Stop a task by the given task ID.
    Args:
        task_id (str): The unique identifier of the task.
    Returns:
        None
    """

    _database_tasks = TaskAPI()
    _database_tasks.update_many(
        query={KEY_TASK_ID: task_id}, update_data={KEY_TASK_STATUS: KEY_TASK_WAIT_STATUS})
    _database_tasks.update_many(
        query={KEY_TASK_SOURCE_ID: task_id}, update_data={KEY_TASK_STATUS: KEY_TASK_WAIT_STATUS})


def query_run_task(query: dict):
    """
    Retrieve a list of tasks from the collection based on the given query, sorted by task weight.

    Args:
        query (dict): A dictionary representing the query conditions for filtering the tasks.

    Returns:
        list: A list of tasks that match the specified query, sorted by task weight in descending order.
    """
    _database_tasks = TaskAPI()
    return _database_tasks.query_list_sort(
        query=query, field={'_id': 0}, limit=1000, skip_no=0, sort_field=KEY_TASK_WEIGHT)



# -*- encoding: utf-8 -*-

import os
import json
import pymongo
from flask_sock import Sock
from flask_cors import CORS
from flask_restful import Api
from flask_session import Session
from flask import Flask, request, session, send_from_directory
from werkzeug.security import generate_password_hash, check_password_hash

from aimaestro.aiagent import OpenAIModel
from aimaestro.abcglobal.key import *
from aimaestro.plugins import *

from aimaestro.taskflux import *

script_path = os.path.dirname(__file__)


def authorize():
    authorization = request.headers.get('Authorization')
    session_id = authorization.split('Bearer ')[-1]
    if session_id in session:
        return session_id
    return False


class APIUser:

    def __init__(self, flask_app):
        self.flask_app = flask_app

        flask_app.add_url_rule('/', view_func=self.index, methods=['GET'])
        flask_app.add_url_rule('/<path:path>', view_func=self.serve_vue, methods=['GET'])
        flask_app.add_url_rule('/assets/<path:filename>', view_func=self.serve_static, methods=['GET'])

        flask_app.add_url_rule('/api/user/info', view_func=self.get_user_info, methods=['GET'])
        flask_app.add_url_rule('/api/user/login', view_func=self.user_login, methods=['POST'])
        flask_app.add_url_rule('/api/user/logout', view_func=self.logout, methods=['POST'])

    def index(self):
        return send_from_directory(self.flask_app.static_folder, 'index.html')

    def serve_vue(self, path):
        return send_from_directory(self.flask_app.static_folder, 'index.html')

    def serve_static(self, filename):
        return send_from_directory(os.path.join(self.flask_app.static_folder, 'assets'), filename)

    @staticmethod
    def user_login():
        request_json = request.get_json()
        username = request_json.get('username')
        password = request_json.get('password')
        user_info = query_user_only(query={'username': username}, field={'_id': 0})
        db_password = user_info.get('password')
        if check_password_hash(db_password, password):
            session_id = generate_password_hash(username)
            session[session_id] = username
            return {'code': 20000, 'msg': 'Login successful', 'data': {'token': session_id}}
        return {'code': 50000, 'msg': 'Invalid username or password'}

    @staticmethod
    def logout():
        session_id = authorize()
        if session_id:
            session.pop(session_id, None)
        return {'code': 20000, 'msg': 'Logout successful'}

    @staticmethod
    def get_user_info():
        session_id = authorize()
        if session_id:
            username = session[session_id]
            user_info = query_user_only(query={'username': username}, field={'_id': 0, 'password': 0})
            return {'code': 20000, 'msg': 'Get user info successful', 'data': user_info}
        return {'code': 50001, 'msg': 'Permission verification failed'}


class APIAiAgent:

    def __init__(self, flask_app, sock_app):
        flask_app.add_url_rule('/api/aiagent/dialogues', view_func=self.dialogues, methods=['GET'])
        flask_app.add_url_rule('/api/aiagent/dialogues', view_func=self.delete_dialogues, methods=['DELETE'])
        flask_app.add_url_rule('/api/aiagent/dialogue_message', view_func=self.dialogue_message, methods=['GET'])

        sock_app.route('/ai_dialogue')(self.ws_ai_dialogue)

    @staticmethod
    def dialogues():
        session_id = authorize()
        if session_id:
            username = session[session_id]
            count, dialogues = query_dialogues(
                query={'username': username, 'status': 'active'},
                field={'_id': 0}, limit=15, skip_no=0
            )
            return {'code': 20000, 'msg': 'Get dialogues successful', 'data': dialogues}
        return {'code': 50001, 'msg': 'Permission verification failed'}

    @staticmethod
    def delete_dialogues():
        session_id = authorize()
        if session_id:
            dialogue_id = request.args.get('dialogue_id')
            dialogues_insert_one(
                query={'dialogue_id': dialogue_id},
                data={'status': 'deleted'}
            )
            dialogues_message_insert_one(
                query={'dialogue_id': dialogue_id},
                data={'status': 'deleted'}
            )

            return {'code': 20000, 'msg': 'Delete dialogues successful'}
        return {'code': 50001, 'msg': 'Permission verification failed'}

    @staticmethod
    def dialogue_message():
        session_id = authorize()
        if session_id:
            dialogue_id = request.args.get('dialogue_id')
            count, message = query_dialogue_message(
                query={'dialogue_id': dialogue_id},
                field={'_id': 0}, limit=200, skip_no=0
            )
            return {'code': 20000, 'msg': 'Get dialogues successful', 'data': message}
        return {'code': 50001, 'msg': 'Permission verification failed'}

    @staticmethod
    def save_dialogue(dialogue_id, username, user_message):
        title = user_message[:10]
        dialogue_data = {
            'dialogue_id': dialogue_id,
            'username': username,
            'title': title,
            'create_time': get_converted_time('%Y-%m-%d %H:%M:%S'),
            'update_time': get_converted_time('%Y-%m-%d %H:%M:%S'),
            'status': 'active'
        }
        dialogues_insert_one(query={'dialogue_id': dialogue_id}, data=dialogue_data)
        return title

    @staticmethod
    def get_history_message(dialogue_id):
        count, data = query_dialogue_message(
            query={'dialogue_id': dialogue_id}, field={'_id': 0}, limit=50, skip_no=0)
        history_message = []
        for m in data:
            if m.get('user_message') is not None:
                history_message.append({'role': 'user', 'content': m.get('user_message')})
            if m.get('chatbot_message') is not None:
                history_message.append({'role': 'assistant', 'content': m.get('chatbot_message')})

        return history_message

    @staticmethod
    def save_user_message(dialogue_id, username, user_message):
        message_id = snowflake_id()
        create_time = get_converted_time('%Y-%m-%d %H:%M:%S')
        message_data = {
            'dialogue_id': dialogue_id,
            'message_id': message_id,
            'username': username,
            'user_message': user_message,
            'create_time': create_time
        }
        dialogues_message_insert_one(query={'dialogue_id': dialogue_id, 'message_id': message_id}, data=message_data)
        return message_id, create_time

    @staticmethod
    def save_chatbot_message(message_id, chatbot_message, reasoning_content):
        update_time = get_converted_time('%Y-%m-%d %H:%M:%S')
        message_data = {
            'message_id': message_id,
            'chatbot_message': chatbot_message,
            'chatbot_reasoner': reasoning_content,
            'update_time': update_time
        }
        dialogues_message_insert_one(query={'message_id': message_id}, data=message_data)
        return update_time

    def ws_ai_dialogue(self, ws):
        ws_data = ws.receive(timeout=300)
        ws_json = json.loads(ws_data)
        authorization = ws_json.get('token')
        session_id = authorization.split('Bearer ')[-1]

        dialogue_id = ws_json.get('dialogue_id')
        user_message = ws_json.get('user_message')
        model = ws_json.get('model', 'deepseek-chat')

        if session_id not in session:
            ws.send(json.dumps({
                'type': 'noPermission', 'msg': 'Permission verification failed, please log in again'}))

        username = session[session_id]
        if dialogue_id == '':
            dialogue_id = snowflake_id()
            title = self.save_dialogue(dialogue_id, username, user_message)
            ws.send(json.dumps({
                "type": "dialogueInit",
                "msg": "dialogue init successful",
                "data": {"title": title, "dialogue_id": dialogue_id}}
            ))

        if not user_message:
            ws.send(json.dumps({"type": "error", "msg": "message is empty"}))
            return None

        system_prompt = """
            你是一个专业的内容生成助手，必须严格遵循以下规则：
            1. 所有回复使用 **标准Markdown语法**
            2. 标题层级使用 #、##、### 
            3. 代码块必须标注语言类型
            4. 列表项使用 - 或 1. 
            5. 禁用纯文本段落`;
            """

        history_message = self.get_history_message(dialogue_id)
        history_message.append({'role': 'user', 'content': user_message})
        history_message.insert(0, {'role': 'system', 'content': system_prompt})

        message_id, create_time = self.save_user_message(
            dialogue_id=dialogue_id, username=username,
            user_message=user_message)

        _client = OpenAIModel(temperature=1.0)
        if model == 'deepseek-reasoner':
            response = _client.get_deepseek_reasoner_response(messages=history_message)
            reasoning_content = response.choices[0].message.reasoning_content
            content = response.choices[0].message.content
        else:
            response = _client.get_deepseek_chat_response(messages=history_message)
            reasoning_content = ''
            content = response.choices[0].message.content

        update_time = self.save_chatbot_message(
            message_id=message_id, chatbot_message=content, reasoning_content=reasoning_content)

        ws.send(json.dumps({
            "type": "chatbotMessage",
            "msg": "ok",
            "data": {
                "reasoning_content": reasoning_content,
                "chatbot_message": content,
                'create_time': create_time,
                'update_time': update_time,
            }}
        ))
        return None


class APINodes:

    def __init__(self, flask_app):
        flask_app.add_url_rule('/api/node/list', view_func=self.get_node_list, methods=['GET'])

    @staticmethod
    def get_node_list():
        session_id = authorize()
        if session_id:
            ipaddr = request.args.get('ipaddr')

            page_size = request.args.get('page_size')
            page = request.args.get('page')
            skip_no = int(page_size) * (int(page) - 1)

            query = {}
            if ipaddr is not None and ipaddr != '':
                query['ipaddr'] = ipaddr

            count, data = query_node_list(query=query, field={"_id": 0}, limit=int(page_size), skip_no=skip_no)
            return {'code': 20000, 'msg': 'Get nodes info successful', 'data': data}
        return {'code': 50001, 'msg': 'Permission verification failed'}


class APIServices:

    def __init__(self, flask_app):
        flask_app.add_url_rule('/api/service/list', view_func=self.get_service_list, methods=['GET'])
        flask_app.add_url_rule('/api/service/update_max_process', view_func=self.update_max_process, methods=['POST'])

    @staticmethod
    def get_service_list():
        session_id = authorize()
        if session_id:
            service_name = request.args.get('service_name')
            service_ipaddr = request.args.get('service_ipaddr')

            page_size = request.args.get('page_size')
            page = request.args.get('page')
            skip_no = int(page_size) * (int(page) - 1)

            query = {}
            if service_name is not None and service_name != '':
                query['service_name'] = service_name

            if service_ipaddr is not None and service_ipaddr != '':
                query['service_ipaddr'] = service_ipaddr

            count, data = query_service_list(query=query, field={"_id": 0}, limit=int(page_size), skip_no=skip_no)
            return {'code': 20000, 'msg': 'Get Service list successful', 'data': data}
        return {'code': 50001, 'msg': 'Permission verification failed'}

    @staticmethod
    def update_max_process():
        session_id = authorize()
        if session_id:
            request_json = request.get_json()
            worker_name = request_json.get('worker_name')
            worker_ipaddr = request_json.get('worker_ipaddr')
            worker_max_process = request_json.get('worker_max_process')

            if worker_max_process is None:
                return {'code': 50000, 'msg': 'Invalid worker max process'}

            worker_max_process = int(worker_max_process)

            update_work_max_process(
                worker_name=worker_name, worker_ipaddr=worker_ipaddr, worker_max_process=worker_max_process)
            return {'code': 20000, 'msg': 'Update worker max process successful'}
        return {'code': 50001, 'msg': 'Permission verification failed'}


class APITask:
    def __init__(self, flask_app):
        flask_app.add_url_rule('/api/task/list', view_func=self.get_task_list, methods=['GET'])
        flask_app.add_url_rule('/api/task/stop', view_func=self.stop_task, methods=['POST'])
        flask_app.add_url_rule('/api/task/retry', view_func=self.retry_task, methods=['POST'])

    @staticmethod
    def get_task_list():
        session_id = authorize()
        if session_id:
            task_id = request.args.get('task_id')
            status = request.args.get('status')
            weight = request.args.get('weight')
            queue_name = request.args.get('queue_name')

            page_size = request.args.get('page_size')
            page = request.args.get('page')
            skip_no = int(page_size) * (int(page) - 1)

            query = {}
            if task_id is not None and task_id != '':
                query['task_id'] = task_id
            if status is not None and status != '':
                query['status'] = status
            if weight is not None and weight != '':
                query['weight'] = weight
            if queue_name is not None and queue_name != '':
                query['queue_name'] = queue_name
            count, data = query_task_list(query=query, field={"_id": 0}, limit=int(page_size), skip_no=skip_no)
            return {'code': 20000, 'msg': 'Get task list successful', 'data': data}
        return {'code': 50001, 'msg': 'Permission verification failed'}

    @staticmethod
    def stop_task():
        session_id = authorize()
        if session_id:
            request_json = request.get_json()
            task_id = request_json.get('task_id')
            task_stop(task_id=task_id)
            return {'code': 20000, 'msg': 'Stop task successful'}
        return {'code': 50001, 'msg': 'Permission verification failed'}

    @staticmethod
    def retry_task():
        session_id = authorize()
        if session_id:
            request_json = request.get_json()
            task_id = request_json.get('task_id')
            task_retry(task_id=task_id)
            return {'code': 20000, 'msg': 'Retry task successful'}
        return {'code': 50001, 'msg': 'Permission verification failed'}


class Management:

    def __init__(self, config):
        self.flask_app = Flask(os.path.dirname(__file__), static_folder=os.path.join(script_path, 'dist'))
        self.config = config
        self.logger = logger(filename=KEY_PROJECT_NAME, task_id='plugin_management')

        self.default_username = self.config.get(KEY_ADMIN_USERNAME, DEFAULT_VALUE_USERNAME)
        self.default_password = self.config.get(KEY_ADMIN_PASSWORD, DEFAULT_VALUE_PASSWORD)

        self.flask_api, self.sock_app = self.initialization()
        self.registry_api()

    def initialization(self):
        CORS(self.flask_app, resources='/*', supports_credentials=True)

        self.flask_app.config['SESSION_TYPE'] = 'mongodb'
        self.flask_app.config['SESSION_MONGODB_COLLECT'] = 'session'
        self.flask_app.config['SESSION_MONGODB_DB'] = KEY_PROJECT_NAME
        self.flask_app.config['SESSION_MONGODB'] = pymongo.MongoClient(self.config.get(KEY_MONGO_CONFIG))
        self.flask_app.config['SOCK_SERVER_OPTIONS'] = {'ping_interval': 25}

        default_user = query_user_only(query={'username': self.default_username}, field={'_id': 0})
        if default_user is None:
            self.logger.info(f'Create default user: {self.default_username}')
            insert_user(data={
                'username': self.default_username,
                'password': generate_password_hash(self.default_password),
                'role': 'admin',
                'create_time': get_converted_time('%Y-%m-%d %H:%M:%S'),
                'update_time': get_converted_time('%Y-%m-%d %H:%M:%S'),
                'status': 'active'
            })

        Session(self.flask_app)
        return Api(self.flask_app), Sock(self.flask_app)

    def registry_api(self):
        APIUser(self.flask_app)
        APIAiAgent(self.flask_app, self.sock_app)
        APINodes(self.flask_app)
        APIServices(self.flask_app)
        APITask(self.flask_app)

    def run(self):
        bind = self.config['management'].get('bind', '0.0.0.0')
        port = int(self.config['management'].get('port', 15673))
        debug = self.config['management'].get('debug', False)
        enable = self.config['management'].get('enable', False)
        if enable:
            self.flask_app.run(host=bind, port=port, debug=debug)



# -*- encoding: utf-8 -*-

from aimaestro.abcglobal.key import *
from aimaestro.plugins import task_insert_one

from aimaestro.taskflux.queue.rabbitmq import send_message
from aimaestro.taskflux.utils.timeformat import get_converted_time
from aimaestro.taskflux.utils.parser import task_required_field_check

__all__ = ['databases_send_message', 'databases_submit_task', 'databases_create_subtask']


def databases_send_message(queue: str, message: dict, weight: int = DEFAULT_VALUE_TASK_WEIGHT) -> str:
    """
        Send a message to the queue.
        Args:
            queue (str): The name of the queue to send the message to.
            message (dict): The message to be sent.
            weight (int): The weight of the message. Default is 1.
        Returns:
            str: The task ID associated with the message.
        This method sends the provided message to the specified queue using the RabbitMQ instance.
    """
    message = task_required_field_check(message=message)
    body = {
        KEY_TASK_BODY: message,
        KEY_TASK_WEIGHT: weight,
        KEY_TASK_QUEUE_NAME: queue,
        KEY_TASK_IS_SUB_TASK: False,
        KEY_TASK_ID: message[KEY_TASK_ID],
        KEY_TASK_STATUS: KEY_TASK_SEND_STATUS,
        KEY_TASK_IS_SUB_TASK_ALL_FINISH: False,
        KEY_TASK_CREATE_TIME: get_converted_time()
    }

    task_insert_one(query={KEY_TASK_ID: body[KEY_TASK_ID]}, data=body)
    send_message(queue=queue, message=message)
    return message[KEY_TASK_ID]


def databases_submit_task(queue: str, message: dict, weight: int = DEFAULT_VALUE_TASK_WEIGHT) -> str:
    """
        Submit a task to the specified queue.
        Args:
            queue (str): The name of the queue to submit the task to.
            message (dict): The message to be submitted as a task.
            weight (int): The weight of the task. Default is 1.
        Returns:
            str: The task ID associated with the submitted task.
        This method submits the provided task to the specified queue using the RabbitMQ instance.
    """
    message = task_required_field_check(message=message)
    body = {
        KEY_TASK_BODY: message,
        KEY_TASK_WEIGHT: weight,
        KEY_TASK_QUEUE_NAME: queue,
        KEY_TASK_IS_SUB_TASK: False,
        KEY_TASK_ID: message[KEY_TASK_ID],
        KEY_TASK_STATUS: KEY_TASK_WAIT_STATUS,
        KEY_TASK_IS_SUB_TASK_ALL_FINISH: False,
        KEY_TASK_CREATE_TIME: get_converted_time()
    }

    task_insert_one(query={KEY_TASK_ID: body[KEY_TASK_ID]}, data=body)
    return message[KEY_TASK_ID]


def databases_create_subtask(source_task_id: str, subtask_queue: str, subtasks: list) -> list:
    """
        Create a subtask for the given task ID.
        Args:
            source_task_id (str): The ID of the task to create a subtask for.
            subtask_queue (str): The name of the queue to create the subtask in.
            subtasks (dict): The subtask to be created.
        Returns:
            str: The subtask ID associated with the created subtask.
        This method creates a subtask for the given task ID using the RabbitMQ instance.
    """
    subtask_ids = []
    for subtask in subtasks:
        message = task_required_field_check(message=subtask)
        message[KEY_TASK_SOURCE_ID] = source_task_id
        body = {
            KEY_TASK_BODY: message,
            KEY_TASK_WEIGHT: DEFAULT_VALUE_TASK_WEIGHT,
            KEY_TASK_QUEUE_NAME: subtask_queue,
            KEY_TASK_SOURCE_ID: source_task_id,
            KEY_TASK_IS_SUB_TASK: True,
            KEY_TASK_ID: message[KEY_TASK_ID],
            KEY_TASK_STATUS: KEY_TASK_WAIT_STATUS,
            KEY_TASK_IS_SUB_TASK_ALL_FINISH: False,
            KEY_TASK_CREATE_TIME: get_converted_time()
        }
        subtask_ids.append(message[KEY_TASK_ID])
        task_insert_one(query={KEY_TASK_ID: body[KEY_TASK_ID]}, data=body)
    return subtask_ids



# -*- encoding: utf-8 -*-
import traceback

from aimaestro.taskflux import *


class RpcFunction(ServiceConstructor):
    """
    Class Name Not modifiable, Define RPC functions
    """
    service_name = 'web_automation'

    def get_service_name(self):
        return {"service_name": self.service_name}


class WorkerFunction(WorkerConstructor):
    """
    Class Name Not modifiable, Work Code
    """

    worker_name = 'web_automation'

    def run(self, data):
        primary_classification = data['primary_classification']
        secondary_classification = data['secondary_classification']

        try:

            if primary_classification == 'selenium_automation':
                from aimaestro.workflows.web_automation import selenium_automation

                selenium_automation.SeleniumAutomation(task=data, log_object=self.logger)

        except Exception as e:
            self.logger.error('{} - {}'.format(e, traceback.print_exc()))



# -*- encoding: utf-8 -*-
import base64
from typing import IO, overload, Union

from Cryptodome.Cipher import PKCS1_v1_5
from Cryptodome.PublicKey import RSA
from Cryptodome import Random

__all__ = ['generate_keys', 'decrypt', 'encrypt']

_key_size = 2048


def _encrypt(plaintext: str, public_key: RSA.RsaKey) -> str:
    """
     Encrypt plaintext using public key.
     :param plaintext: plaintext to be encrypted
     :param public_key: public key used for encryption
     :return: encrypted text in base64-encoded string format
    """
    cipher = PKCS1_v1_5.new(public_key)
    encrypted = cipher.encrypt(plaintext.encode('utf-8'))
    return base64.b64encode(encrypted).decode('utf-8')


def _decrypt(ciphertext: str, private_key: RSA.RsaKey) -> str:
    """
    Decrypt ciphertext using private key.
    :param ciphertext: ciphertext to be decrypted
    :param private_key: private key used for decryption
    :return: decrypted text in base64-encoded string format
    """
    cipher = PKCS1_v1_5.new(private_key)
    decoded = base64.b64decode(ciphertext)
    decrypted = cipher.decrypt(decoded, None)
    return decrypted.decode('utf-8')


def generate_keys(key_size: int = _key_size) -> tuple[bytes, bytes]:
    """
    Generate random keys.
    :param key_size: key size in bits
    :return: tuple of public key and private key in bytes
    """
    random_generator = Random.new().read
    private_key = RSA.generate(key_size, random_generator)
    public_key = private_key.publickey()
    return public_key.export_key(), private_key.export_key()


@overload
def decrypt(ciphertext: str, private_key: RSA.RsaKey) -> str: ...


@overload
def decrypt(ciphertext: str, private_key: bytes) -> str: ...


@overload
def decrypt(ciphertext: str, private_key: str) -> str: ...


@overload
def decrypt(ciphertext: str, private_key: IO[bytes]) -> str: ...


def decrypt(ciphertext: str, private_key: Union[RSA.RsaKey, bytes, str, IO[bytes]]) -> str:
    """
    Decrypt ciphertext using private key.
    :param ciphertext: ciphertext to be decrypted
    :param private_key: private key used for decryption
    :return: decrypted text in base64-encoded string format
    """
    if isinstance(private_key, RSA.RsaKey):
        return _decrypt(ciphertext, private_key)
    elif isinstance(private_key, bytes):
        key = RSA.import_key(private_key)
        return _decrypt(ciphertext, key)
    elif isinstance(private_key, str):
        key = RSA.import_key(base64.b64decode(private_key.encode('utf-8')))
        return _decrypt(ciphertext, key)
    elif hasattr(private_key, 'read'):
        key_data = private_key.read()
        key = RSA.import_key(key_data)
        return _decrypt(ciphertext, key)
    else:
        raise ValueError("Unsupported private key format")


@overload
def encrypt(plaintext: str, public_key: RSA.RsaKey) -> str: ...


@overload
def encrypt(plaintext: str, public_key: bytes) -> str: ...


@overload
def encrypt(plaintext: str, public_key: str) -> str: ...


@overload
def encrypt(plaintext: str, public_key: IO[bytes]) -> str: ...


def encrypt(plaintext: str, public_key: Union[RSA.RsaKey, bytes, str, IO[bytes]]) -> str:
    """
    Encrypt plaintext using public key.
    :param plaintext: plaintext to be encrypted
    :param public_key: public key used for encryption
    :return: encrypted text in base64-encoded string format
    """
    if isinstance(public_key, RSA.RsaKey):
        return _encrypt(plaintext, public_key)
    elif isinstance(public_key, bytes):
        key = RSA.import_key(public_key)
        return _encrypt(plaintext, key)
    elif isinstance(public_key, str):
        key = RSA.import_key(base64.b64decode(public_key.encode('utf-8')))
        return _encrypt(plaintext, key)
    elif hasattr(public_key, 'read'):
        key_data = public_key.read()
        key = RSA.import_key(key_data)
        return _encrypt(plaintext, key)
    else:
        raise ValueError("Unsupported public key format")



# -*- encoding: utf-8 -*-
import os
import time
from aimaestro.taskflux import *

from aimaestro.abcglobal import *
from aimaestro.plugins import SeleniumOperations


class SeleniumAutomation:

    def __init__(self, task, log_object):
        self.task = task
        self.log_object = log_object

        self.log_object.info(task)

        self.task_id = task['task_id']
        self.all_save_screenshot = task.get('all_save_screenshot')

        self.temporary_dir = os.path.join(ROOT_PATH, 'temporary_dir', 'web_automation', self.task_id)
        os.makedirs(self.temporary_dir, exist_ok=True)

        self.operations = SeleniumOperations(
            log_object=log_object,
            browser=task['browser'],
            wait_time=task['wait_time'],
            params=task['params'],
            width=task['width'],
            height=task['height']
        )
        self.run()

    def run(self):
        setup = 0

        assert_status = True
        for operation in self.task['operations']:
            if assert_status is False:
                break

            st = get_converted_time_float()

            describe = operation.get('describe')
            operation_type = operation.get('operation_type')
            value = operation.get('value')
            sleep = operation.get('sleep')
            locators = operation.get('locators')

            if operation_type == 'open_url':
                self.operations.open_url(url=value)
                self.log_object.info(f'Open url: {value}')

            if operation_type == 'save_screenshot':
                save_path = os.path.join(str(self.temporary_dir), f'{setup}.png')
                self.operations.save_screenshot(path=save_path)
                operation['screenshot_path'] = save_path

            if operation_type == 'input_text':
                element = self.operations.find_element(locators=locators)
                self.operations.input_text(element=element, keys=value)

            if operation_type == 'send_keys':
                element = self.operations.find_element(locators=locators)
                self.operations.send_keys(element=element, keys=value)

            if operation_type == 'click':
                element = self.operations.find_element(locators=locators)
                self.operations.click(element=element)

            if self.all_save_screenshot and operation_type != 'save_screenshot':
                save_path = os.path.join(str(self.temporary_dir), f'{setup}.png')
                self.operations.save_screenshot(path=save_path)

            time.sleep(sleep)
            if 'asserts' in operation:
                for _assert in operation['asserts']:
                    assert_type = _assert.get('assert_type')
                    assert_value = _assert.get('assert_value')
                    assert_locators = _assert.get('locators')

                    element_value = None
                    if assert_type == 'title':
                        element_value = self.operations.get_title()

                    if assert_type == 'text':
                        element = self.operations.find_element(locators=assert_locators)
                        element_value = element.text

                    if assert_type == 'selected':
                        element = self.operations.find_element(locators=assert_locators)
                        element_value = self.operations.assert_is_selected(element)

                    if assert_type == 'displayed':
                        element = self.operations.find_element(locators=assert_locators)
                        element_value = self.operations.assert_is_displayed(element)

                    if assert_type == 'attribute':
                        element = self.operations.find_element(locators=assert_locators)
                        expression = _assert.get('expression')
                        element_value = self.operations.get_attribute(element, attribute=expression)

                    if element_value != assert_value:
                        operation['assert_result'] = False
                        assert_status = False
                        self.log_object.info(f'[{describe}] assert failed')
                        break
                    else:
                        operation['assert_result'] = True

            et = get_converted_time_float()
            self.log_object.info(f'[{describe}] cost: {et - st}')
            setup += 1

        self.task['assert_status'] = assert_status
        self.log_object.info(self.task)



# -*- encoding: utf-8 -*-

import time
import pytz
import pickle
from pymongo import MongoClient
from threading import Thread, Event
from datetime import datetime, timedelta

from aimaestro.abcglobal.key import *
from aimaestro.taskflux.logger.logger import initialization_logger, logger
from aimaestro.taskflux.scheduler.cronparser import CronParser

__all__ = [
    'initialization_scheduler',
    'scheduler_add_job',
    'scheduler_remove_job',
    'scheduler_start',
    'scheduler_stop'
]


def initialization_scheduler(config: dict):
    """
    Initialize the scheduler.
    :param config: The configuration dictionary.
    """
    Scheduler(config=config)


def scheduler_add_job(job_id, cron_str, func_object, timezone="UTC", args=None, kwargs=None):
    """
    Add a job to the scheduler.
    :param job_id: The ID of the job.
    :param cron_str: The cron string defining the schedule.
    :param func_object: The function object to execute.
    :param timezone: The timezone for the schedule.
    :param args: Additional arguments to pass to the function.
    :param kwargs: Additional keyword arguments to pass to the function.
    """
    Scheduler().add_job(job_id, cron_str, func_object, timezone, args, kwargs)


def scheduler_remove_job(job_id):
    """
    Remove a job from the scheduler.
    :param job_id: The ID of the job to remove.
    """
    Scheduler().remove_job(job_id)


def scheduler_start():
    """
    Start the scheduler.
    """
    Scheduler().start()


def scheduler_stop():
    """
    Stop the scheduler.
    """
    Scheduler().stop()


class Scheduler:
    _instance = None

    _db = None
    _jobs = None
    _locks = None
    _client = None
    _thread = None
    _running = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Scheduler, cls).__new__(cls)
            cls._instance.__initialize(*args, **kwargs)
        return cls._instance

    def __initialize(self, config: dict):
        initialization_logger(config=config)
        self.logger = logger(filename=KEY_PROJECT_NAME, task_id='scheduler')

        self._client = MongoClient(config[KEY_MONGO_CONFIG])
        self._db = self._client[KEY_PROJECT_NAME]
        self._jobs = self._db.scheduled_jobs
        self._locks = self._db.job_locks
        self._running = Event()
        self._thread = None

    def start(self):
        self._running.set()
        self._thread = Thread(target=self._run_loop)
        self._thread.start()

    def stop(self):
        self._running.clear()
        self._thread.join()

    def _run_loop(self):
        while self._running.is_set():
            now = datetime.now(pytz.utc)
            due_jobs = self._jobs.find({
                "enabled": True,
                "next_run_time": {"$lte": now}
            })
            for job in due_jobs:
                self._execute_job(job)
            time.sleep(1)

    def _execute_job(self, job):
        lock_expire = datetime.now(pytz.utc) + timedelta(seconds=30)
        lock_result = self._locks.update_one(
            {"job_id": job["_id"], "expire_at": {"$lt": datetime.now(pytz.utc)}},
            {"$set": {"expire_at": lock_expire}},
            upsert=True
        )
        if lock_result.modified_count > 0 or lock_result.upserted_id:
            try:
                func = pickle.loads(job["func"])
                Thread(target=self._run_task, args=(func, job)).start()

                cron_parser = CronParser(
                    cron_str=job["cron"],
                    timezone=job.get("timezone", "UTC")
                )
                next_run = cron_parser.get_next_run(datetime.now(pytz.utc))

                self._jobs.update_one(
                    {"_id": job["_id"]},
                    {"$set": {"next_run_time": next_run, "last_run_time": datetime.now(pytz.utc)}}
                )
            except Exception as e:
                self.logger.error(f"Task execution failed: {job['_id']} - {str(e)}")
            finally:
                self._locks.delete_one({"job_id": job["_id"]})

    def _run_task(self, func, job):
        try:
            func(*job["args"], **job["kwargs"])
        except Exception as e:
            self.logger.error(f"Task execution exception: {job['_id']} - {str(e)}")

    def add_job(self, job_id, cron_str, func_object, timezone="UTC", args=None, kwargs=None):
        existing_job = self._jobs.find_one({"_id": job_id})
        if existing_job:
            self.logger.warning(f"Job '{job_id}' already exists. Skipping insertion.")
            return

        cron_parser = CronParser(cron_str, timezone=timezone)
        next_run = cron_parser.get_next_run(datetime.now(pytz.utc))

        self._jobs.insert_one({
            "_id": job_id,
            "cron": cron_str,
            "func": pickle.dumps(func_object),
            "timezone": timezone,
            "args": args or [],
            "kwargs": kwargs or {},
            "enabled": True,
            "next_run_time": next_run,
            "last_run_time": None
        })

    def remove_job(self, job_id):
        self._jobs.delete_one({"_id": job_id})



# -*- encoding: utf-8 -*-
import os
import yaml

__all__ = ['ROOT_PATH', 'GlobalVar']

ROOT_PATH = os.getcwd()


class GlobalVar:
    _instance = None

    global_config = {}
    taskflux_config = {}

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(GlobalVar, cls).__new__(cls)
            cls._instance.__initialize(*args, **kwargs)
        return cls._instance

    def __initialize(self, config_file: str, current_dir: str):
        self.config_file = config_file
        self._load_global_config()
        self._load_taskflux_config()

        global ROOT_PATH
        ROOT_PATH = current_dir
        print('ROOT_PATH == ', ROOT_PATH)

    def _load_global_config(self):
        with open(self.config_file, "r", encoding="utf-8") as f:
            self.global_config = yaml.safe_load(f)

    def _load_taskflux_config(self):
        self.taskflux_config = {
            'ROOT_PATH': ROOT_PATH,
            'RABBITMQ_CONFIG': 'amqp://{}:{}@{}:{}'.format(
                self.global_config['RabbitMQ']['username'],
                self.global_config['RabbitMQ']['password'],
                self.global_config['RabbitMQ']['host'],
                self.global_config['RabbitMQ']['port']
            ),
            'MONGODB_CONFIG': 'mongodb://{}:{}@{}:{}'.format(
                self.global_config['Database']['mongodb']['username'],
                self.global_config['Database']['mongodb']['password'],
                self.global_config['Database']['mongodb']['host'],
                self.global_config['Database']['mongodb']['port']
            ),
            **self.global_config
        }



# -*- encoding: utf-8 -*-
from openai import OpenAI
from aimaestro.abcglobal import *


class OpenAIModel:

    def __init__(self, temperature: float = 1.0):
        self._temperature = temperature

        self._client = None

        self._app_key = None
        self._base_url = None
        self._config_key = 'OpenAI'

        self._global_config = GlobalVar().global_config

        self._get_config()
        self._get_client()

    def get_deepseek_chat_response(self, messages: list, stream=False, model: str = 'deepseek-chat', ):
        """
        Get the response from the deepseek-chat model.
        param prompt:
        """
        response = self._client.chat.completions.create(
            model=model,
            messages=messages,
            stream=stream
        )
        return response

    def get_deepseek_reasoner_response(self, messages: list, stream=False, model: str = 'deepseek-reasoner', ):
        """
        Get the response from the deepseek-reasoner model.
        param prompt:
        """
        response = self._client.chat.completions.create(
            model=model,
            messages=messages,
            stream=stream
        )
        return response

    def _get_client(self):
        """
        Get the OpenAI client.
        :return:
        """
        if self._client is None:
            self._client = OpenAI(api_key=self._app_key, base_url=self._base_url)
        return self._client

    def _get_config(self):
        """
        Get the configuration for the OpenAI model.
        :return:
        """
        self._app_key = self._global_config.get(self._config_key)['deepseek'].get('api_key')
        self._base_url = self._global_config.get(self._config_key)['deepseek'].get('base_url')



# -*- encoding: utf-8 -*-
import copy

from selenium import webdriver
from selenium.webdriver import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import NoSuchElementException, TimeoutException

from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager

__all__ = ["SeleniumOperations"]


class SeleniumOperations:

    def __init__(self, log_object, browser: str = 'chrome', wait_time: int = 30,
                 params: dict = None, width: int = 1920, height: int = 1080):
        self.log_object = log_object
        self.width = width
        self.height = height

        self.options = Options()

        if params is None or len(params) == 0:
            params = [
                '--headless',
                '--disable-gpu',
                '--lang=zh-CN.UTF-8',
                '--force-device-scale-factor=0.90'
            ]

        for p in params:
            self.options.add_argument(p)

        if browser == "chrome":
            self.driver = webdriver.Chrome(
                service=webdriver.ChromeService(
                    executable_path=ChromeDriverManager().install()
                ),
                options=self.options
            )
        if browser == 'firefox':
            self.driver = webdriver.Firefox(
                service=webdriver.FirefoxService(
                    executable_path=GeckoDriverManager().install()
                ),
                options=self.options
            )
        if browser == 'edge':
            self.driver = webdriver.Edge(
                service=webdriver.EdgeService(
                    executable_path=EdgeChromiumDriverManager().install()
                ),
                options=self.options
            )

        self.wait_time = wait_time
        self.driver.implicitly_wait(self.wait_time)

    @staticmethod
    def _get_keys(key: str):
        return getattr(Keys, key)

    def find_element(self, locators: list):
        """
        find element by locators
        :param locators: [{'Xpath': 'x'}, {'ID': '1'}, ...]
        :return: WebElement
        """
        original_implicit_wait = copy.deepcopy(self.wait_time)
        self.driver.implicitly_wait(1)

        try:
            by_value_pairs = []
            for locator in locators:
                locator_type = list(locator.keys())[0].upper()
                locator_value = list(locator.values())[0]
                by = getattr(By, locator_type)
                by_value_pairs.append((by, locator_value))

            def _any_locator_visible(driver):
                for _by, value in by_value_pairs:
                    try:
                        _element = driver.find_element(_by, value)
                        if _element.is_displayed():
                            return _element
                    except NoSuchElementException:
                        continue
                return False

            element = WebDriverWait(driver=self.driver, timeout=original_implicit_wait).until(_any_locator_visible)
            return element

        except TimeoutException:
            raise NoSuchElementException(f"all not find : {locators}")
        finally:
            self.driver.implicitly_wait(original_implicit_wait)

    def open_url(self, url: str):
        self.driver.get(url)
        self.driver.set_window_size(self.width, self.height)

    def save_screenshot(self, path: str):
        self.driver.save_screenshot(path)

    def send_keys(self, element, keys: str):
        element.send_keys(self._get_keys(keys))

    def get_title(self):
        return self.driver.title

    @staticmethod
    def get_attribute(element, attribute: str):
        return element.get_attribute(attribute)

    @staticmethod
    def input_text(element, keys: str):
        element.send_keys(keys)

    @staticmethod
    def click(element):
        element.click()

    @staticmethod
    def select(element, selector: str):
        element.select_by_value(selector)

    @staticmethod
    def assert_is_selected(element):
        return element.is_selected()

    @staticmethod
    def assert_is_displayed(element):
        return element.is_displayed()

    def close(self):
        self.driver.close()



# -*- encoding: utf-8 -*-

import json
import logging

from aimaestro.plugins import *
from aimaestro.abcglobal.key import *

from aimaestro.taskflux.ameta import *
from aimaestro.taskflux.main.main import *
from aimaestro.taskflux.cipher.rsa import *
from aimaestro.taskflux.utils.parser import *
from aimaestro.taskflux.logger.logger import *
from aimaestro.taskflux.utils.network import *
from aimaestro.taskflux.queue.rabbitmq import *
from aimaestro.taskflux.utils.timeformat import *
from aimaestro.taskflux.rpc_proxy.rpc_proxy import *
from aimaestro.taskflux.rpc_proxy.decorator import *
from aimaestro.taskflux.interface.interface import *
from aimaestro.taskflux.generateId.snowflake import *
from aimaestro.taskflux.scheduler.scheduler import *

__all__ = [
    'services_registry',
    'services_start',
    'generate_keys',
    'decrypt',
    'encrypt',
    'load_config',
    'task_required_field_check',
    'logger',
    'get_ipaddr',
    'is_port_open',
    'send_message',
    'receive_message',
    "get_date_time_obj",
    "format_converted_time",
    "get_converted_time",
    "get_yes_today",
    "get_yesterday_date",
    "convert_timestamp_to_timezone",
    "get_converted_timestamp",
    "get_date_list",
    "get_week_num",
    "get_current_week",
    "is_timestamp_within_days",
    "convert_timestamp_to_timezone_obj",
    "convert_timestamp_to_timezone_str",
    'get_converted_time_float',
    'remote_call',
    'generate_unique',
    'proxy_call',
    'rpc',
    'service_running',
    'databases_send_message',
    'databases_submit_task',
    'databases_create_subtask',
    'query_node_list',
    'node_insert_one',
    'query_service_list',
    'service_insert_one',
    'update_work_max_process',
    'query_task_list',
    'task_insert_one',
    'query_task_status_by_task_id',
    'task_stop',
    'task_retry',
    'query_run_task',
    'query_worker_running_number',
    'worker_push_process',
    'worker_pull_process',
    'snowflake_id',
    'scheduler_add_job',
    'scheduler_remove_job',
    'scheduler_start',
    'scheduler_stop',
    'ServiceConstructor',
    'WorkerConstructor',
    'initialization_taskflux',
    'loguru'
]


def initialization_taskflux(config: dict, is_cipher: bool = False):
    """
        TaskFlux is a singleton class designed to manage and schedule RPC services.
        It initializes various components such as logging, message queues, RPC proxies,
        and databases based on the provided configuration.

        Example usage:
            import os
            current_dir = os.path.dirname(os.path.abspath(__file__))

            config = {
                'MONGODB_CONFIG': 'mongodb://scheduleAdmin:scheduleAdminPasswrd@127.0.0.1:27017',
                'RABBITMQ_CONFIG': 'amqp://scheduleAdmin:scheduleAdminPasswrd@127.0.0.1:5672',
                'ROOT_PATH': current_dir
            }

            initialization_taskflux(config=config)
    """
    if KEY_ROOT_PATH not in config:
        raise Exception('Error ROOT_PATH not in config')

    if is_cipher:
        cipher_config = encrypt(plaintext=config[KEY_CIPHER_CIPHERTEXT], public_key=config[KEY_CIPHER_PUBLIC_KEY])
        config_dict = json.loads(cipher_config)
        config_dict[KEY_ROOT_PATH] = config[KEY_ROOT_PATH]
    else:
        config_dict = config

    initialization_global_attr(config_dict)


def loguru(filename: str = None, task_id: str = None) -> logging.Logger:
    """
    Get a logger instance for logging.
    Args:
        filename (str): The name of the log file.
        task_id (str, optional): The ID of the task. Defaults to None.
    Returns:
        logging: The logger instance.
    """

    if task_id and filename:
        return logger(filename=filename, task_id=task_id)
    return logger(filename=KEY_PROJECT_NAME, task_id=KEY_PROJECT_NAME)



# -*- encoding: utf-8 -*-
import os
import re
import time
import logging

__all__ = ['NmapScan']


def _get_ports(nmap_output: str):
    result = []
    patterns = r'(\d+/tcp|udp)\s+(.+)'
    for line in nmap_output.split('\n'):
        matches = re.findall(patterns, line)
        for match in matches:
            port = match[0]
            if len(port.split('/')) != 2:
                continue

            port_number = port.split('/')[0]
            protocol = port.split('/')[1]

            pattern = r"(?!closed\b)(\b\w+\b)\s+(\w+)\s*(.*)"
            submatch = re.findall(pattern, match[1])[0]

            state = submatch[0]
            service = submatch[1]

            if len(submatch) == 3:
                version = submatch[2]
            else:
                version = ''

            result.append({
                'port': port_number,
                'protocol': protocol,
                'state': state,
                'service': service,
                'version': version
            })
    return result


def _get_other_info(nmap_output: str):
    result = {}
    patterns = {
        'mac_address': r"MAC Address: (.+)",
        'device_type': r"Device type: (.+)",
        'running': r"Running: (.+)",
        'os_cpe': r"OS CPE: (.+)",
        'os_details': r"OS details: (.+)",
        'network_distance': r"Network Distance: (.+)",
        'service_info': r"Service Info: (.+)"
    }
    for key, pattern in patterns.items():
        match = re.search(pattern, nmap_output)
        if match:
            result[key] = match.group(1)
    return result


class NmapScan:

    def __init__(self, ip: str, min_rate: int = 1000, logger: logging = logging):
        self.logger = logger

        self.ip = ip
        self.min_rate = min_rate
        self.max_rate = min_rate * 10
        self.result = {}

        self.target_state = False
        self._target_survival_detection()
        if self.target_state is False:
            self._target_survival_detection_pn()

        self.logger.info('Target {} state: {}'.format(self.ip, self.target_state))
        if self.target_state is True:
            self._target_information_collection()

    @staticmethod
    def _get_command(command: str):
        if os.name == 'nt':
            return command
        return '{} {}'.format('sudo', command)

    def _target_survival_detection(self):
        start_time = time.time()
        self.logger.info('Start nmap -P/U/E/M/A {} {}'.format(self.ip, start_time))

        commands = [
            'nmap -PU -PE -PP -PM --min-rate {} {}'.format(self.min_rate, self.ip),
            'nmap -PS -PA -PE -PP -PM --min-rate {} {}'.format(self.min_rate, self.ip),
        ]
        for command in commands:
            result = os.popen(self._get_command(command)).read()
            for line in result.split('\n'):
                if 'open' in line:
                    self.target_state = True
                    self.logger.info('End nmap -P/U/E/M/A {} {}'.format(self.ip, time.time() - start_time))
                    return True
        self.logger.info('End nmap -P/U/E/M/A {} {}'.format(self.ip, time.time() - start_time))
        return False

    def _target_survival_detection_pn(self):
        start_time = time.time()
        self.logger.info('Start nmap -Pn {} {}'.format(self.ip, start_time))
        command = 'nmap -Pn --min-rate {} {}'.format(self.min_rate, self.ip)
        result = os.popen(self._get_command(command)).read()
        for line in result.split('\n'):
            if 'open' in line:
                self.target_state = True
                self.logger.info('End nmap -Pn {} {}'.format(self.ip, time.time() - start_time))
                return True
        self.logger.info('End nmap -Pn {} {}'.format(self.ip, time.time() - start_time))
        return False

    def _target_information_collection(self):
        start_time = time.time()
        self.logger.info('Start nmap -sTV/sUV -O {} {}'.format(self.ip, start_time))

        commands = {
            'tcp': 'nmap -sTV -O --min-rate {} {}'.format(self.min_rate, self.ip),
            'udp': 'nmap -sU --min-rate {} {}'.format(self.max_rate, self.ip),
        }

        results = {}
        for protocol, command in commands.items():
            nmap_output = os.popen(self._get_command(command)).read()

            ports = _get_ports(nmap_output=nmap_output)
            other_info = _get_other_info(nmap_output=nmap_output)
            other_info['open_ports'] = ports
            results[protocol] = other_info

        self.logger.info('End nmap -sTV/sUV -O {} {}'.format(self.ip, time.time() - start_time))
        results['ip'] = self.ip
        results['duration'] = time.time() - start_time
        self.result = results



# -*- encoding: utf-8 -*-
from aimaestro.abcglobal.key import *
from aimaestro.plugins._mongodb.mongo_client import MongoClient

__all__ = [
    'query_dialogues',
    'dialogues_insert_one',
    'query_dialogue_message',
    'dialogues_message_insert_one',
    'query_one_dialogue'
]


class Dialogues(MongoClient):
    """
        DatabaseNodes class is used to manage database dialogues.
    """

    _table_name = TABLE_NAME_DIALOGUES


class DialogueMessage(MongoClient):
    """
        DatabaseNodes class is used to manage database dialogue message.
    """
    _table_name = TABLE_NAME_DIALOGUE_MESSAGE


def query_dialogues(query: dict, field: dict, limit: int, skip_no: int):
    """
    Query the list of nodes from the database.
    Args:
        query (dict): The query criteria.
        field (dict): The fields to include in the result.
        limit (int): The maximum number of results to return.
        skip_no (int): The number of results to skip.
    Returns:
        list: The list of services.
    """
    _api = Dialogues()
    return _api.query_list_sort(query=query, field=field, limit=limit, skip_no=skip_no)


def query_one_dialogue(dialogue_id: str):
    """
    Query the list of nodes from the database.
    Args:
        dialogue_id (dict): The query criteria.
    Returns:
        list: The list of services.
    """
    _api = Dialogues()
    return _api.query_one(query={'dialogue_id': dialogue_id}, field={'_id': 0})


def dialogues_insert_one(query: dict, data: dict):
    """
    Insert a single document into the collection.

    Args:
        data (dict): A dictionary containing the data to insert.
        query (dict): A dictionary specifying the query criteria.
    """
    _api = Dialogues()
    _api.update_many(query=query, update_data=data, upsert=True)


def query_dialogue_message(query: dict, field: dict, limit: int, skip_no: int,
                           sort_field: str = 'create_time', sort: int = 1):
    """
    Query the list of nodes from the database.
    Args:
        query (dict): The query criteria.
        field (dict): The fields to include in the result.
        limit (int): The maximum number of results to return.
        skip_no (int): The number of results to skip.
        sort_field (str): The field to sort by.
        sort (int): The sort order. 1 for ascending, -1 for descending.
    Returns:
        list: The list of services.
    """
    _api = DialogueMessage()
    return _api.query_list_sort(
        query=query, field=field, limit=limit, skip_no=skip_no, sort_field=sort_field, sort=sort)


def dialogues_message_insert_one(query: dict, data: dict):
    """
    Insert a single document into the collection.

    Args:
        data (dict): A dictionary containing the data to insert.
        query (dict): A dictionary specifying the query criteria.
    """
    _api = DialogueMessage()
    _api.update_many(query=query, update_data=data, upsert=True)



# -*- encoding: utf-8 -*-
import time

from .taskflux import *

from .abcglobal import *
from .management import Management


class AiMaestro:

    def __init__(self, config_file: str, current_dir):
        GlobalVar(config_file=config_file, current_dir=current_dir)

        initialization_taskflux(config=GlobalVar().taskflux_config)

    @staticmethod
    def registry_services(services: list):
        services_registry(services=services)

    @staticmethod
    def start_management():
        Management(config=GlobalVar().taskflux_config).run()

    def start_services(self):
        services_start()
        time.sleep(10)
        self.start_management()
